.. -*- rst -*-

=====
FIP64
=====

.. note:: Currently this extension ``fip64`` is only available for networking-midonet.

This extension ``fip64`` provides
`NAT64 <https://tools.ietf.org/html/rfc6146>`__
functionality by allowing to associate IPv6 floating IPs to IPv4 fixed IPs.
(Without this extension, floating IPs are limited to IPv4.)

This extension does not introduce any resources or attributes.
