.. -*- rst -*-

============================
Router Interface floating IP
============================

.. note:: Currently this extension ``router-interface-fip`` is only available for networking-midonet.

This extension ``router-interface-fip`` indicates the ability to
associate floating IPs to internal interfaces of a router.
(Without this extension, floating IPs can be associated only to
the gateway interface of a router.)

This extension does not introduce any resources or attributes.
